@@ -112,6 +112,7 @@ enum hw_dec_formats {
   DEC_HAS_VC1_L,
   DEC_HAS_RV,
   DEC_HAS_AVS2,
+  DEC_HAS_XVID,
   DEC_HAS_CSC,
   DEC_FORMATS_MAX
 };
@@ -179,7 +180,7 @@ typedef enum {
   V4L2_DAEMON_CODEC_DEC_VC1_L,
   V4L2_DAEMON_CODEC_DEC_RV,
   V4L2_DAEMON_CODEC_DEC_AVS2,
-
+  V4L2_DAEMON_CODEC_DEC_XVID,
   V4L2_DAEMON_CODEC_UNKNOW_TYPE,
 } v4l2_daemon_codec_fmt;
 
@@ -468,6 +469,8 @@ typedef struct v4l2_daemon_enc_h26x_cmd {
   uint32_t vuiColorPrimaries;
   uint32_t vuiTransferCharacteristics;
   uint32_t vuiMatrixCoefficients;
+
+  uint32_t idrHdr;
 } v4l2_daemon_enc_h26x_cmd;
 
 typedef struct v4l2_daemon_enc_jpeg_cmd {
