@@ -164,7 +164,8 @@ int vsi_dec_get_codec_format(struct vsi_v4l2_dev_info *hwinfo) {
        ((hw_cfg[0].mpeg4_support != 0) ? (1 << DEC_HAS_H263) : 0) |
        ((hw_cfg[0].jpeg_support != 0) ? (1 << DEC_HAS_JPEG) : 0) |
        ((hw_cfg[0].avs_support != 0) ? (1 << DEC_HAS_AVS2) : 0) |
-       ((hw_cfg[0].rv_support != 0) ? (1 << DEC_HAS_RV) : 0));
+       ((hw_cfg[0].rv_support != 0) ? (1 << DEC_HAS_RV) : 0) |
+       ((hw_cfg[0].mpeg4_support != 0) ? (1 << DEC_HAS_XVID) : 0));
 #endif
   g2_dec_format = (((hw_cfg[1].hevc_support != 0) ? (1 << DEC_HAS_HEVC) : 0) |
                    ((hw_cfg[1].vp9_support != 0) ? (1 << DEC_HAS_VP9) : 0));
@@ -255,6 +256,7 @@ void vsi_dec_register_fsm_table(v4l2_dec_inst *h) {
       break;
     case V4L2_DAEMON_CODEC_DEC_H263:
     case V4L2_DAEMON_CODEC_DEC_MPEG4:
+    case V4L2_DAEMON_CODEC_DEC_XVID:
       h->vsi_v4l2m2m_decoder = (void *)&vsidaemon_mpeg4_v4l2m2m_decoder;
       break;
     case V4L2_DAEMON_CODEC_DEC_VC1_G:
@@ -596,6 +598,22 @@ re_get:
   return 0;
 }
 
+void vsi_dec_send_picconsumed_info(v4l2_dec_inst *h, i32 in_idx, i32 out_idx) {
+    vsi_v4l2_msg msg = {0};
+
+    msg.error = 0;
+    msg.seq_id = NO_RESPONSE_SEQID;
+    msg.size = sizeof(struct v4l2_daemon_dec_buffers);
+    msg.cmd_id = V4L2_DAEMON_VIDIOC_PICCONSUMED;
+    msg.inst_id = h->instance_id;
+    msg.params.dec_params.io_buffer.inbufidx = in_idx;
+    msg.params.dec_params.io_buffer.outbufidx = out_idx;
+    send_notif_to_v4l2(pipe_fd, &msg,
+                        sizeof(struct vsi_v4l2_msg_hdr) +
+                            sizeof(struct v4l2_daemon_dec_buffers));
+}
+
+
 static void vsi_dec_handle_inpkt_cnt(v4l2_dec_inst *h) {
   /* If current input packet doesn't have picture decoded,
    * send out a specific event out. It is for Android CTS,
@@ -604,28 +622,25 @@ static void vsi_dec_handle_inpkt_cnt(v4l2_dec_inst *h) {
   if (h->dec_inpkt_pic_decoded_cnt == h->dec_pic_id &&
       h->dec_inpkt_ignore_picconsumed_event == 0) {
     BUFFER *p = NULL;
-    i32 inbufidx = INVALID_IOBUFF_IDX;
 
     p = bufferlist_get_buffer(h->bufferlist_input);
-
     if (p == NULL) {
         HANTRO_LOG(HANTRO_LEVEL_ERROR, "Inst[%lx]: SHOULD GET INPUT BUFFER.\n",
-                   h->instance_id);
+                    h->instance_id);
         ASSERT(0);
     } else {
-        inbufidx = p->dec_cmd.io_buffer.inbufidx;
+      vsi_dec_send_picconsumed_info(h, p->dec_cmd.io_buffer.inbufidx, INVALID_IOBUFF_IDX);
     }
-    vsi_v4l2_msg msg = {0};
-    msg.error = 0;
-    msg.seq_id = NO_RESPONSE_SEQID;
-    msg.size = sizeof(struct v4l2_daemon_dec_buffers);
-    msg.cmd_id = V4L2_DAEMON_VIDIOC_PICCONSUMED;
-    msg.inst_id = h->instance_id;
-    msg.params.dec_params.io_buffer.inbufidx = inbufidx;
-    msg.params.dec_params.io_buffer.outbufidx = INVALID_IOBUFF_IDX;
-    send_notif_to_v4l2(pipe_fd, &msg,
-                       sizeof(struct vsi_v4l2_msg_hdr) +
-                           sizeof(struct v4l2_daemon_dec_buffers));
+  }
+
+  if (h->discard_dbp_count > 0) {
+    i32 i;
+
+    for (i = 0; i < h->discard_dbp_count; i++) {
+      vsi_dec_send_picconsumed_info(h, INVALID_IOBUFF_IDX, INVALID_IOBUFF_IDX);
+      TSManagerSend2(h->tsm, NULL);
+    }
+    h->discard_dbp_count = 0;
   }
 }
 
@@ -1671,6 +1686,11 @@ int vsi_dec_seek2(v4l2_dec_inst *h, vsi_v4l2_msg *v4l2_msg) {
 int vsi_dec_seek3(v4l2_dec_inst *h, vsi_v4l2_msg *v4l2_msg) {
   ASSERT(h->dec_cur_state == DAEMON_DEC_STATE_SEEK);
 
+  if (h->dec_output_state2 != 0) {
+    HANTRO_LOG(HANTRO_LEVEL_DEBUG, "reset dec_output_state2\n");
+    h->dec_output_state2 = 0;
+  }
+
   if (h->existed_dpb_nums)
     h->dec_output_state = DAEMON_QUEUE_STATE_ON;
   else
