@@ -1033,16 +1033,3 @@ int32_t calculate_level(v4l2_daemon_codec_fmt     format, int32_t width, int32_t
     return leveIdx;
 }
 
-int32_t is_input_level_ilegal(v4l2_daemon_codec_fmt format, int32_t width, int32_t height,
-                                    uint32_t frameRateNum, uint32_t frameRateDenom, int32_t level_idx_input) {
-    uint32_t sample_per_picture = width * height;
-    uint32_t sample_per_second = sample_per_picture * frameRateNum / frameRateDenom;
-
-    uint32_t sample_per_picture_limit = getMaxPicSize(format, level_idx_input);
-    unsigned long long sample_per_second_limit = getMaxSBPS(format, level_idx_input);
-    if((sample_per_picture > sample_per_picture_limit) || (sample_per_second > sample_per_second_limit)) {
-        return 1;
-    }
-    else
-        return 0;
-}
