@@ -273,13 +273,13 @@ static int init_encoder_h264(v4l2_enc_inst *h,
   if (enc_params->specific.enc_h26x_cmd.avclevel != DEFAULTLEVEL)
     cfg->level = (H264EncLevel)enc_params->specific.enc_h26x_cmd.avclevel;
 
-  int32_t levelIdx = getLevelIdxH264(cfg->level);
+  int32_t levelIdx = getLevelIdxH264(cfg->level), minLvl;
 
-  if(1 == is_input_level_ilegal(h->codec_fmt, cfg->width, cfg->height,
-                                cfg->frameRateNum, cfg->frameRateDenom, levelIdx)) {
-      levelIdx = calculate_level(h->codec_fmt, cfg->width, cfg->height,
-                                cfg->frameRateNum, cfg->frameRateDenom, enc_params->general.bitPerSecond, enc_params->specific.enc_h26x_cmd.profile);
-      cfg->level = getLevelH264(levelIdx);
+  minLvl = calculate_level(h->codec_fmt, cfg->width, cfg->height,
+                 cfg->frameRateNum, cfg->frameRateDenom, enc_params->general.bitPerSecond,
+                 enc_params->specific.enc_h26x_cmd.profile);
+   if(levelIdx < minLvl) {
+      cfg->level = getLevelH264(minLvl);
       send_warning_orphan_msg(h->instance_id, WARN_LEVEL);
   }
 
@@ -670,6 +670,7 @@ static int set_coding_control_h264(v4l2_enc_inst *h,
     return DAEMON_ERR_ENC_INTERNAL;
   }
 
+  codingCfg->idrHeader = enc_params->specific.enc_h26x_cmd.idrHdr;
   if (enc_params->specific.enc_h26x_cmd.sliceSize != -1)
     codingCfg->sliceSize = enc_params->specific.enc_h26x_cmd.sliceSize;
   if (enc_params->specific.enc_h26x_cmd.enableCabac != -1)
