@@ -1324,7 +1324,7 @@ static int get_config_from_cmd_h2(v4l2_enc_inst *h,
            IS_HEVC(enc_params->general.codecFormat))
     cfg->level = (VCEncLevel)enc_params->specific.enc_h26x_cmd.hevclevel;
 
-  int32_t levelIdx = 0;
+  int32_t levelIdx = 0, minLvl;
   if(IS_HEVC(enc_params->general.codecFormat)) {
       levelIdx = getlevelIdxHevc(cfg->level);
   }
@@ -1351,11 +1351,11 @@ static int get_config_from_cmd_h2(v4l2_enc_inst *h,
         cfg->profile <= VCENC_H264_HIGH_PROFILE)
       cfg->profile = VCENC_HEVC_MAIN_PROFILE;
   }
-  if(is_input_level_ilegal(h->codec_fmt, cfg->width, cfg->height,
-        cfg->frameRateNum, cfg->frameRateDenom, levelIdx)) {
-      levelIdx = calculate_level(h->codec_fmt, cfg->width, cfg->height,
-                                cfg->frameRateNum, cfg->frameRateDenom, enc_params->general.bitPerSecond, cfg->profile);
-      cfg->level = getLevel(h->codec_fmt, levelIdx);
+  minLvl = calculate_level(h->codec_fmt, cfg->width, cfg->height,
+                  cfg->frameRateNum, cfg->frameRateDenom,
+                  enc_params->general.bitPerSecond, cfg->profile);
+  if(levelIdx < minLvl) {
+      cfg->level = getLevel(h->codec_fmt, minLvl);
       send_warning_orphan_msg(h->instance_id, WARN_LEVEL);
   }
 
@@ -2200,9 +2200,8 @@ void encoder_get_input_h2(v4l2_enc_inst *h, v4l2_daemon_enc_params *enc_params,
 
   encIn->vui_timing_info_enable = 1;
 
-  if (encIn->codingType == VCENC_INTRA_FRAME && !if_config)
-    encIn->resendVPS = encIn->resendSPS = encIn->resendPPS =
-        1;  // default enabe this value
+  if (encIn->codingType == VCENC_INTRA_FRAME && !if_config && enc_params->specific.enc_h26x_cmd.idrHdr)
+    encIn->resendVPS = encIn->resendSPS = encIn->resendPPS = 1;  // default enabe this value
   else
     encIn->resendVPS = encIn->resendSPS = encIn->resendPPS = 0;
 
@@ -2214,6 +2213,8 @@ void encoder_get_input_h2(v4l2_enc_inst *h, v4l2_daemon_enc_params *enc_params,
   else
     encIn->timeIncrement = 1;
   if (enc_params->specific.enc_h26x_cmd.force_idr) {
+    encIn->poc = 0;
+    encIn->last_idr_picture_cnt = encIn->picture_cnt;
     encIn->codingType = VCENC_INTRA_FRAME;
     encIn->bIsIDR = 1;
     h->next_pic_type = (int)VCENC_INTRA_FRAME;
