@@ -42,6 +42,7 @@
 #include <unistd.h>
 #include <errno.h>
 #include <signal.h>
+#include <time.h>
 
 #include "daemon_instance.h"
 #include "hash_table.h"
@@ -52,6 +53,7 @@ typedef struct zombie_inst{
   struct zombie_inst *next;
 } zombie_inst;
 
+static int64_t zombie_delaytime = 1000000000;
 FILE *vsidaemonstdlog = NULL;
 static HashTable ht;
 static pthread_t maintid;
@@ -100,21 +102,39 @@ static void check_instance(v4l2_daemon_inst *cur_inst_p) {
   }
 }
 
+static v4l2_daemon_inst *find_zombieinst(pthread_t tid)
+{
+    zombie_inst *pz = hzinst;
+
+    while (pz) {
+        if (pz->pinst->tid == tid)
+            return pz->pinst;
+        pz = pz->next;
+    }
+    return NULL;	
+}
+
 static void signalhandler(int signo) {
   v4l2_daemon_inst *inst_p;
   pthread_t tid = pthread_self();
   HANTRO_LOG(HANTRO_LEVEL_ERROR, "%s : sig: %d(%s), thread: %lX \r\n",
              __FUNCTION__, signo, strsignal(signo), pthread_self());
 
-  if ((inst_p = hash_table_find_bytid(&ht, tid)) != NULL) {
+  inst_p = hash_table_find_bytid(&ht, tid);
+  if (inst_p == NULL)
+    inst_p = find_zombieinst(tid);
+  if (inst_p != NULL) {
     HANTRO_LOG(HANTRO_LEVEL_ERROR, "%s: catch exception for ctx %lx !!!!!\r\n",
                __FUNCTION__, inst_p->instance_id);
     if (inst_p->flag & (1 << INST_CATCH_EXCEPTION)) {
       HANTRO_LOG(HANTRO_LEVEL_ERROR, "%s: repeatedly catch exception for ctx %lx !!!!!\r",
         __FUNCTION__, inst_p->instance_id);
       inst_p->flag |= (1 << INST_FATAL_ERROR);
-    } else
+    } else {
+      if (signo != SIGUSR1)
+          inst_p->flag |= (1 << INST_CATCH_EXCEPTION);
       siglongjmp(inst_p->sigbuf, 1);
+    }
   } else if (tid == maintid) {
     HANTRO_LOG(HANTRO_LEVEL_ERROR, "%s: catch exception for main!!!!!\r\n",
                __FUNCTION__);
@@ -131,7 +151,7 @@ static void signalhandler(int signo) {
 static int register_sighandler(void) {
   int i;
   static const int siglist[] = {
-      SIGHUP, SIGINT,  SIGILL,  SIGABRT, SIGBUS,
+      SIGHUP, SIGINT,  SIGILL,  SIGUSR1, SIGABRT, SIGBUS,
       SIGFPE, SIGSEGV, SIGTERM, SIGTSTP, SIGTTIN,
   };
   for (i = 0; i < sizeof(siglist) / sizeof(siglist[0]); i++) {
@@ -167,6 +187,7 @@ static int add_zombieinst(v4l2_daemon_inst *pinst, struct timespec *pstarttime)
         return -1;
     pz->next = NULL;
     pz->pinst = pinst;
+    pz->pinst->flag |= (1 << INST_ZOMBIE);
     pz->starttime = *pstarttime;    
     if (hzinst == NULL)
         hzinst = pz;
@@ -185,29 +206,33 @@ static int zombie_timeout(struct timespec *before)
     clock_gettime(CLOCK_REALTIME, &after);
     elapse = ((int64_t)after.tv_sec - (int64_t)before->tv_sec) * (int64_t)1000000000
          + ((int64_t)after.tv_nsec - (int64_t)before->tv_nsec);
-    return (elapse >= (int64_t)1000000000); //over 1 sec
+    return (elapse >= zombie_delaytime); //over 1 sec
 }
 
 static void check_zombieinst(void)
 {
     zombie_inst *pz = hzinst, *pn, *prev = NULL;
-    int ret, done;
+    int ret = 0, done;
 
     while (pz) {
         done = 0;
-        pn = pz->next;        
-        ret = sem_trywait(&pz->pinst->sem_done);
-        if (ret == 0)
-            done = 1;
-        else if (zombie_timeout(&pz->starttime)) {
-            kill(pz->pinst->tid, SIGKILL);
+        pn = pz->next;
+        if (pz->pinst->flag & (1 << INST_DEAD))
             done = 1;
+        else {
+            ret = sem_trywait(&pz->pinst->sem_done);
+            if (ret == 0)
+                done = 1;
+            else if (zombie_timeout(&pz->starttime))
+                pthread_kill(pz->pinst->tid, SIGUSR1);
         }
         if (done) {
             HANTRO_LOG(HANTRO_LEVEL_WARNING, "release zombie inst %lx with %d\n", pz->pinst->instance_id, ret);
-            inst_destroy(pz->pinst);        
-            free(pz->pinst);
-            free(pz);
+            if (done) {
+                inst_destroy(pz->pinst);
+                free(pz->pinst);
+                free(pz);
+            }
             if (prev == NULL)
                 hzinst = pz = pn;
             else
@@ -221,20 +246,45 @@ static void check_zombieinst(void)
 
 static void removeall_zombieinst(void)
 {
-    zombie_inst *pz = hzinst, *pn;
+    int i;
+    zombie_inst *pz = hzinst, *pn, *prev;
+    struct timespec interval, rem;
+
+    if (hzinst == NULL)
+        return;
     while (pz) {
-        pn = pz->next;
-        HANTRO_LOG(HANTRO_LEVEL_WARNING, "release zombie inst %lx in tail\n", pz->pinst->instance_id);
-        kill(pz->pinst->tid, SIGKILL);
-        if (!(pz->pinst->flag & (1 << INST_FATAL_ERROR)))
-            inst_destroy(pz->pinst);
-        free(pz->pinst);
-        free(pz);
-        pz = pn;
+        pz->pinst->flag |= (1 << INST_FORCEEXIT);
+        pthread_kill(pz->pinst->tid, SIGUSR1);
+        pz = pz->next;
+    }
+    interval.tv_sec = 0;
+    interval.tv_nsec = 100;
+    for (i = 0; i < 100; i++) {
+        nanosleep(&interval, &rem);
+        pz = hzinst;
+        prev = NULL;
+        while (pz) {
+            pn = pz->next;
+            if (pz->pinst->flag & (1 << INST_DEAD)){
+                HANTRO_LOG(HANTRO_LEVEL_WARNING, "release zombie inst %lx in tail\n", pz->pinst->instance_id);
+                inst_destroy(pz->pinst);
+                free(pz->pinst);
+                free(pz);
+                if (prev == NULL)
+                    hzinst = pz = pn;
+                else
+                    prev->next = pz = pn;
+            } else {
+                prev = pz;
+                pz = pn;
+            }
+        }
+        if (hzinst == NULL)
+            return;
     }
-    hzinst = NULL;
 }
 
+
 /**
  * @brief Main thread.
  * @return an int value.
@@ -309,7 +359,7 @@ int32_t main(int32_t argc, char **argv) {
   struct vsi_v4l2_dev_info hwinfo = {0};
   get_codec_info(&hwinfo);
 
-  v4l2_daemon_inst *cur_inst_p;
+  v4l2_daemon_inst *cur_inst_p = NULL;
   hash_table_init(&ht);
   maintid = pthread_self();
   HANTRO_LOG(HANTRO_LEVEL_INFO, "main thread id = %lx\n", maintid);
@@ -373,7 +423,7 @@ int32_t main(int32_t argc, char **argv) {
         if (vsi_v4l2_cmd_p.msg.cmd_id == V4L2_DAEMON_VIDIOC_STREAMOFF
           || vsi_v4l2_cmd_p.msg.cmd_id == V4L2_DAEMON_VIDIOC_DESTROY_DEC
           || vsi_v4l2_cmd_p.msg.cmd_id == V4L2_DAEMON_VIDIOC_EXIT) {
-          kill(cur_inst_p->tid, SIGKILL);
+          pthread_kill(cur_inst_p->tid, SIGKILL);
           //inst_destroy is not reliable in this condition
           hash_table_remove(&ht, vsi_v4l2_cmd_p.msg.inst_id, 1);
         }
@@ -397,18 +447,13 @@ int32_t main(int32_t argc, char **argv) {
             semtime.tv_nsec += 50000000;	//20ms should be enough for normal exit
             ret = sem_timedwait(&cur_inst_p->sem_done, &semtime);
             if (ret != 0) {
-                HANTRO_LOG(HANTRO_LEVEL_WARNING, "wait %lx exit timeout\n", cur_inst_p->instance_id);
-                if (vsi_v4l2_cmd_p.msg.cmd_id == V4L2_DAEMON_VIDIOC_EXIT) {
-                  kill(cur_inst_p->tid, SIGKILL);
+              HANTRO_LOG(HANTRO_LEVEL_WARNING, "wait %lx exit timeout\n", cur_inst_p->instance_id);
+
+              hash_table_remove(&ht, vsi_v4l2_cmd_p.msg.inst_id, 0);
+              if (add_zombieinst(cur_inst_p, &semtime)) {
                   inst_destroy(cur_inst_p);
-                  hash_table_remove(&ht, vsi_v4l2_cmd_p.msg.inst_id, 1);
-                } else {
-                  hash_table_remove(&ht, vsi_v4l2_cmd_p.msg.inst_id, 0); //make it zombie
-                  if (add_zombieinst(cur_inst_p, &semtime)) {
-                      inst_destroy(cur_inst_p);
-                      free(cur_inst_p);
-                  }
-                }
+                  free(cur_inst_p);
+              }
             } else {
               inst_destroy(cur_inst_p);
               hash_table_remove(&ht, vsi_v4l2_cmd_p.msg.inst_id, 1);
@@ -429,6 +474,7 @@ int32_t main(int32_t argc, char **argv) {
     vsi_v4l2_cmd_p.msg.inst_id = 0;  // make sure this value is read from pipe.
   }
 exit:
+  close(pipe_fd);
   removeall_zombieinst();
   if (vsidaemonstdlog) fclose(vsidaemonstdlog);
   return 0;
@@ -445,10 +491,12 @@ void *worker_threads(void *arg) {
   if (ret != 0) {
     // send error msg to driver
     HANTRO_LOG(HANTRO_LEVEL_ERROR, " %lx catch exception \n", inst_id);
-    if (h->instance_id != NO_RESPONSE_SEQID) {
+    if (h->flag & (1 << INST_ZOMBIE))
+        h->flag |= (1 << INST_DEAD);
+    if (h->flag & (1 << INST_FORCEEXIT))
+        goto label_worker_done;
+    if (h->instance_id != NO_RESPONSE_SEQID)
       send_fatalerror_orphan_msg(h->instance_id, DAEMON_ERR_SIGNAL_CONFIG);
-    }
-    h->flag |= (1 << INST_CATCH_EXCEPTION);
     goto label_worker_done;
   }
 
@@ -488,7 +536,6 @@ label_worker_done:
   sem_post(&h->sem_done);
   HANTRO_LOG(HANTRO_LEVEL_DEBUG, "worker_threads finish, instance id %lx\n",
              h->instance_id);
-
   return NULL;
 }
 
@@ -519,6 +566,7 @@ static void inst_set_format(v4l2_daemon_inst *h) {
     case V4L2_DAEMON_CODEC_DEC_VC1_L:
     case V4L2_DAEMON_CODEC_DEC_RV:
     case V4L2_DAEMON_CODEC_DEC_AVS2:
+    case V4L2_DAEMON_CODEC_DEC_XVID:
       h->codec_mode = DAEMON_MODE_DECODER;
       h->func.create = vsi_create_decoder;
       break;
@@ -634,7 +682,6 @@ int inst_destroy(v4l2_daemon_inst *h) {
 
   // object_heap_destroy(&(h->cmds));
   destroy_heap(&(h->cmds), object_heap_free);
-  memset(h, 0, sizeof(v4l2_daemon_inst));
   HANTRO_LOG(HANTRO_LEVEL_DEBUG, "inst_destroy finish\n");
   return 0;
 }
